! This file is part of HiggsBounds
!  -KW
!********************************************************
module output
!********************************************************	
 contains	

 !********************************************************	
 subroutine setup_output
 ! Creates the key to the process numbers used
 ! (by the way, note that these numbers are dependent on the 
 ! setting of whichexpt)
 ! Sets 'rep'. This is the number of processes that will be checked
 ! against the experimentally measured limit
 ! e.g. if rep=2, the process with the highest stat sens will be checked, 
 ! but so will the process with the 2nd highest stat.sens.
 ! (can be useful, despitethe statistical intepretation of results from
 !  the 2nd highest process being hazy)
 ! Allocates the array 'res', which holds stuff to do with output
 !********************************************************
  use usefulbits, only : ndat,inputmethod, & !input
                       & rep, & !output
                       & res    !allocated
  implicit none
  !-----------------------------------internal
  integer :: i
  !-------------------------------------------
  

  select case(inputmethod)
  case('datfile')
   !create Key to process numbers in file Key.dat	
   call createKey 
   rep=1
  case('website') 
   rep=3          !if change this, also need to change subroutine weboutput
                  !in module extra_bits_for_web	 
  case('subrout')
   !create Key to process numbers in file Key.dat	
#ifndef WEBVERSION
   call createKey 
#endif
   rep=1          !if change this, also need to change subroutine run_HiggsBounds_effC
  end select
	
  allocate(res(ndat))
  do i=1,ndat
   allocate(res(i)%chan(rep))
   allocate(res(i)%obsratio(rep))
   allocate(res(i)%allowed95(rep))
   allocate(res(i)%ncombined(rep))	 
  enddo
	
 end subroutine setup_output
 
 !********************************************************	
 subroutine do_output
 ! Writes output to file or screen, depending on whether 
 ! inputmethod='datfile' or inputmethod='website'
 !******************************************************** 
  use usefulbits, only : res,theo,ndat,inputmethod,nH, &
          &              n_additional, & !input   
	  &              whichexpt,whichinput,pr,listprocesses,infile1  
  use extra_bits_for_web
  use S95tables	
  use input, only : use_additional_file
  
  implicit none
  !-----------------------------------internal
  integer :: i,jj,x,n,n_sum
  character(LEN=1) :: nHchar
  character(LEN=3) :: addchar,tempchar
  character(LEN=8) :: createdate
  character(LEN=10) :: createtime
  character(LEN=19) :: createdateandtime
  character(LEN=100) :: format43  
  character(LEN=500) :: columndescription
  type(listprocesses) :: proc  
  character(LEN=150):: descrip 
  !-------------------------------------------
  n_sum=0
  do i=1,ndat
   n_sum=n_sum+res(i)%allowed95(1)
  enddo

  if(n_sum.le.(-ndat))then ! i.e. no good data sets
   !write(*,*)'Check that Mh>1 and the sum of the branching ratios for each Higgs is <=1'
   write(*,*)'Check that Mh>1 for at east one data set'
   stop 'no valid data sets : see standard output'
  endif
  
  write(nHchar,'(I1)')nH
  write(addchar,'(I3)')n_additional
  
  if(use_additional_file)then
   format43='(1I10,'//nHchar//'G16.6,2I6,G16.6,1I6,'//trim(addchar)//'G16.6)'  	
  else
   format43='(1I10,'//nHchar//'G16.6,2I6,G16.6,1I6)'
  endif  

  select case(inputmethod)	
  case('datfile')	 
   open(12,file=trim(infile1)//"HiggsBounds_results.dat") 
   
   call date_and_time(createdate,createtime)

   createdateandtime=createdate(7:8)//'.' &
         &  //createdate(5:6)//'.' &
         &  //createdate(1:4)//' at '&
         &  //createtime(1:2)//':' & 
         &  //createtime(3:4)  
   
   write(12,*)'# generated with HiggsBounds on '//createdateandtime
   write(12,*)'# settings: '//whichexpt//', '//whichinput   
   write(12,*)'#'
   write(12,*)'# column abbreviations' 
   write(12,*)'#   n          : line id of input' 
   write(12,*)'#   Mh(i)      : Higgs boson masses in GeV'
   write(12,*)'#   HBresult   : scenario allowed flag (1: allowed, 0: excluded, -1: unphysical)'      
   write(12,*)'#   chan       : most sensitive channel (see below). chan=0 if no channel applies'          
   write(12,*)'#   obsratio   : ratio [sig x BR]_model/[sig x BR]_limit (<1: allowed, >1: excluded)' 
   write(12,*)'#   ncomb      : number of Higgs bosons combined in most sensitive channel'
   write(12,*)'#   additional : optional additional data stored in <prefix>additional.dat (e.g. tan beta)'     
   write(12,*)'#'   
   write(12,*)'# channel numbers used in this file'  
   
   do x=1,ubound(pr,dim=1)
    n=0
    do jj=1,ndat   
      if(x.eq.res(jj)%chan(1))n=n+1
    enddo
    if(n.ge.1)then
      proc=pr(x)   
      write(tempchar,'(I3)')x          
      call outputproc(proc,12,descrip)
      write(12,*)'#         '//trim(tempchar)//' :'//trim(descrip)       
    endif     
   enddo   
   
   write(12,*)'# (for full list of processes, see Key.dat)'   
   write(12,*)'#'     
      
   columndescription='#cols:  n       Mh(1)'
   do i=2,nH
    write(tempchar,'(I3)')i
    columndescription=trim(columndescription)//'           Mh('//trim(adjustl(tempchar))//')  '
   enddo                                        
   columndescription=trim(columndescription)//'  HBresult  chan    obsratio     ncomb '

   if(use_additional_file)then
    do i=1,n_additional
     write(tempchar,'(I3)')i
     columndescription=trim(columndescription)//'   additional('//trim(adjustl(tempchar))//')   '
    enddo     
   endif

   write(12,*)trim(columndescription) 
   write(12,*)'#'  

   if(use_additional_file)then
    do jj=1,ndat   	         	    	    	                 
       write(12,fmt=trim(format43))jj,(theo(jj)%Mh(i)        ,i=1,nH), &
              &	res(jj)%allowed95(1),res(jj)%chan(1),res(jj)%obsratio(1),res(jj)%ncombined(1), &
	      &  (theo(jj)%additional(i),i=1,n_additional)
    enddo  
   else
    do jj=1,ndat   	         	    	    	                 
       write(12,fmt=trim(format43))jj,(theo(jj)%Mh(i)        ,i=1,nH), &
              &	res(jj)%allowed95(1),res(jj)%chan(1),res(jj)%obsratio(1),res(jj)%ncombined(1)
    enddo  
   endif


   close(12)
  case('website')         	
    do jj=1,ndat 
      call weboutput(res(jj))
    enddo								
  end select  
  								
 end subroutine do_output
 !********************************************************	
 subroutine createKey
 ! writes the key to the procss numbers
 ! (by the way, note that the process numbers are dependent on the 
 ! setting of whichexpt)
 !********************************************************
  use S95tables
  use usefulbits, only : nH,ntot,pr,infile1 !input
  implicit none
  !-----------------------------------internal
  integer :: i,j,n   
  character(LEN=25) :: tableno
  character(LEN=150):: descrip 
  !-------------------------------------------	
	
  open(21,file=trim(infile1)//'Key.dat')

  132   format('***********  Key to Process Numbers  ************',/ &
              ,'LEP table numbers are taken from hep-ex/0602042  ',/ &
              ,'(apart from 22,23,24 which were not published)   ',/ )
		  		  
  131   format('*************************************************',/ &
              ,'                ',      A      ,'                ',/ )	
	      	  
  130   format('process ',1I4)		  
  
  133   format(' ----------------------------------------------- ')

  write(21,132)
  write(21,130)0
  write(21,*)' no process applies'

 	
  do n=1,ntot    
   i=pr(n)%findi
   j=pr(n)%findj
   select case(pr(n)%ttype)
   case(1)
    tableno=trim(S95_t1(pr(n)%tlist)%label)
   case(2)
    tableno=trim(S95_t2(pr(n)%tlist)%label)  
   end select
	  
   if((j.eq.1).and.(i.eq.1))write(21,131)trim(tableno)
	  
   write(21,130)n	  	  

   if(pr(n)%ttype.eq.1)then
    call outputproc(pr(n),21,descrip)
    write(21,*)trim(descrip)
   elseif(pr(n)%ttype.eq.2)then
    if((i.eq.j).and.(S95_t2(pr(n)%tlist)%needs_Mh2_gt_2Mh1))then
    ! these processes involve Br(hj->hj+hj) and so never exist
      write(21,*)' ***'		     
    else
      call outputproc(pr(n),21,descrip)
      write(21,*)trim(descrip)
    endif 
   else 
    stop 'error in subroutine createKey'
   endif
	  
   if((i.eq.nH).and.(j.ne.nH))write(21,133)	         
  enddo     
	
  close(21)
	
 end subroutine createKey
!******************************************************************

end module output
!******************************************************************
